#include "main.h"
#include "tty.h"
#include "dscf1_low.h"

#define PMP_RESOLUTION 0x1d
#define PMP_FIN 0x33
#define PMP_STD 0x17
#define PMP_ECM 0x08

#define PMP_COMMENT 0x34

#define PMP_TAKE_YEAR 0x4c
#define PMP_TAKE_MONTH 0x4d
#define PMP_TAKE_DATE 0x4e
#define PMP_TAKE_HOUR 0x4f
#define PMP_TAKE_MINUTE 0x50
#define PMP_TAKE_SECOND 0x51

#define PMP_EDIT_YEAR 0x54
#define PMP_EDIT_MONTH 0x55
#define PMP_EDIT_DATE 0x56
#define PMP_EDIT_HOUR 0x57
#define PMP_EDIT_MINUTE 0x58
#define PMP_EDIT_SECOND 0x59

#define PMP_SPEED 0x66

#define PMP_FLASH 0x76

static int totalbytes=0;

int
make_jpeg_comment(u_char *buf, u_char *jpeg_comment)
{
    int i, cur = 0;
    int reso, shutter;

    struct resolution {
	int reso_val;
	char *reso_conv;
    } reso_tab[] = {
	{PMP_FIN, "fine"},
	{PMP_STD, "standard"},
	{PMP_ECM, "economy"},
	{0,       "unknown"},
    }; 

    struct sh_speed {
	int spd_val;
	char *spd_conv;
    } sh_speed_tab[] = {
	{0x0123, "1/7.5"},
	{0x0187, "1/15"},
	{0x01eb, "1/30"},
	{0x024f, "1/60"},
	{0x0298, "1/100"},
	{0x031d, "1/250"},
	{0x0381, "1/500"},
	{0x03e5, "1/1000"},
	{0,      "unknown"},
    };

    jpeg_comment[0] = 0xff;
    jpeg_comment[1] = 0xd8;
    jpeg_comment[2] = 0xff;
    jpeg_comment[3] = 0xfe;

    /* resolution */
    reso = *(buf+PMP_RESOLUTION);

    for (i=0; ;i++)
	if ((reso == reso_tab[i].reso_val) || (reso_tab[i].reso_val == 0)) {
	    cur = 6 + sprintf(&jpeg_comment[6], "Resolution: %s\n", reso_tab[i].reso_conv);
	    break;
	}

    /* shutter speed */
    shutter = (int)(buf[PMP_SPEED] << 8) | buf[PMP_SPEED+1];

    for (i=0; ;i++)
	if ((shutter == sh_speed_tab[i].spd_val) || (sh_speed_tab[i].spd_val == 0)) {
	    cur += sprintf(&jpeg_comment[cur], "Shutter-speed: %s\n", sh_speed_tab[i].spd_conv);
	    break;
	}

    /* PMP comment */
    if (*(buf+PMP_COMMENT))
	cur += sprintf(&jpeg_comment[cur], "Comment: %s\n", (char *)(buf+PMP_COMMENT));

    /* taken date */
    if (*(buf+PMP_TAKE_YEAR) == 0xff)
	cur += sprintf(&jpeg_comment[cur], "Date-Taken: ----/--/-- --:--:--\n");

    else {
	cur += sprintf(&jpeg_comment[cur],
			    "Date-Taken: %d/%02d/%02d %02d:%02d:%02d\n",
			    1900+(*(buf+PMP_TAKE_YEAR)), *(buf+PMP_TAKE_MONTH),
			    *(buf+PMP_TAKE_DATE), *(buf+PMP_TAKE_HOUR), *(buf+PMP_TAKE_MINUTE),
			    *(buf+PMP_TAKE_SECOND)); 
    }

    /* edited date */
    if (*(buf+PMP_EDIT_YEAR) == 0xff)
	cur += sprintf(&jpeg_comment[cur], "Date-Edited: ----/--/-- --:--:--\n");
    else
	cur += sprintf(&jpeg_comment[cur],
			    "Date-Edited: %d/%02d/%02d %02d:%02d:%02d\n",
			    1900+(*(buf+PMP_EDIT_YEAR)), *(buf+PMP_EDIT_MONTH),
			    *(buf+PMP_EDIT_DATE), *(buf+PMP_EDIT_HOUR), *(buf+PMP_EDIT_MINUTE),
			    *(buf+PMP_EDIT_SECOND));

    /* use flash? */
    if (*(buf+PMP_FLASH) != 0)
	cur += sprintf(&jpeg_comment[cur], "Flash: on\n");

    /* insert total jpeg comment length */
    jpeg_comment[4] = (u_char)((cur - 4) >> 8);
    jpeg_comment[5] = (u_char)(cur - 4);

    return cur;
}

long
GetPicture()
{
    u_long filelen;
    static u_char buf[0x1000];
    long len;
    char name[64];
    
    if (!totalbytes) {
	sprintf(name, "/PIC_CAM/PIC00000/PSN%05d.PMP", pics[onpic].pic_ind);

	CamReset();
	CamOk();
	CamStatus(0);

	filelen = CamFinfo(name);
	if (filelen == 0)
	    return(-1);

	if (CamFopen(name) != 0)
	    return(-2);

	len = CamFread(buf, 126);
	if (len < 126) {
	    CamFclose();
	    return(0);
	}
	pics[onpic].fp = OpenGrabFile();
	fwrite(buf+0x500, 1, make_jpeg_comment(buf, buf+0x500), pics[onpic].fp);
	totalbytes = 126;
	return 2+filelen;
    }

    if ((len = CamFread(buf, 0x500)) != 0) {
	if (len < 0) {
	    CamFclose();
	    return(-3);
	}
	totalbytes += len;
	fwrite(buf, sizeof(u_char), (size_t) len, pics[onpic].fp);
	return -3-totalbytes;
    }
    fclose(pics[onpic].fp);
    CamFclose();

    totalbytes = 0;
    onpic++;

    return 0;
}

int
InitPics(int verbose)
{
    u_char buf[4096];
    char name[64];
    int i, n;

    sprintf(name, "/PIC_CAM/PIC00000/PIC_INF.PMF");
    if (CamGetdata(name, buf, 0) < 0) {
	printf("Can not get PIC_INF.PMF from camera\n");
	return -1;
    }

    n = buf[26] * 256 + buf[27];
    if (n != totalpics) {
	printf("total pics returned by PIC_INF.PMF is not correct. (%d != %d)\n", n, totalpics);
	return -1;
    }

    pics = calloc(totalpics, sizeof(Pic));
    for (i=0; i<totalpics; i++) {
	pics[i].pic_ind = buf[0x420 + 0x10 * i + 3];
	pics[i].rotate  = buf[0x420 + 0x10 * i + 5];
	pics[i].protect = buf[0x420 + 0x10 * i + 14];
	if (verbose)
	    printf("%2d) PSN%05d.PMP, rotated %3d degrees, %s protected\n", i+1, pics[i].pic_ind, 
		   (pics[i].rotate == 0) ? 0 : (pics[i].rotate == 4) ? 270 : (pics[i].rotate == 8) ? 180 : 90,
		   pics[i].protect ? "   " : "not");
    }
    return n;
}
